@@ -126,7 +126,7 @@ static void format_kbyte_out(request_rec *r, apr_off_t kbytes)
         ap_rprintf(r, "%.1f GB", (float) kbytes / MBYTE);
 }
 
-static void show_time(request_rec *r, apr_interval_time_t tsecs)
+static void show_time(request_rec *r, apr_uint32_t tsecs)
 {
     int days, hrs, mins, secs;
 
@@ -183,7 +183,8 @@ static int status_handler(request_rec *r)
 {
     const char *loc;
     apr_time_t nowtime;
-    apr_interval_time_t up_time;
+    apr_uint32_t up_time;
+    ap_loadavg_t t;
     int j, i, res, written;
     int ready;
     int busy;
@@ -194,7 +195,7 @@ static int status_handler(request_rec *r)
     long req_time;
     int short_report;
     int no_table_report;
-    worker_score *ws_record;
+    worker_score *ws_record = apr_palloc(r->pool, sizeof *ws_record);
     process_score *ps_record;
     char *stat_buffer;
     pid_t *pid_buffer, worker_pid;
@@ -306,7 +307,7 @@ static int status_handler(request_rec *r)
         for (j = 0; j < thread_limit; ++j) {
             int indx = (i * thread_limit) + j;
 
-            ws_record = ap_get_scoreboard_worker_from_indexes(i, j);
+            ap_copy_scoreboard_worker(ws_record, i, j);
             res = ws_record->status;
 
             if ((i >= max_servers || j >= threads_per_child)
@@ -391,10 +392,9 @@ static int status_handler(request_rec *r)
     /* up_time in seconds */
     up_time = (apr_uint32_t) apr_time_sec(nowtime -
                                ap_scoreboard_image->global->restart_time);
+    ap_get_loadavg(&t);
 
     if (!short_report) {
-        ap_loadavg_t t;
-
         ap_rputs(DOCTYPE_HTML_3_2
                  "<html><head>\n"
                  "<title>Apache Status</title>\n"
@@ -405,6 +405,8 @@ static int status_handler(request_rec *r)
                   ")</h1>\n\n", NULL);
         ap_rvputs(r, "<dl><dt>Server Version: ",
                   ap_get_server_description(), "</dt>\n", NULL);
+        ap_rvputs(r, "<dt>Server MPM: ",
+                  ap_show_mpm(), "</dt>\n", NULL);
         ap_rvputs(r, "<dt>Server Built: ",
                   ap_get_server_built(), "\n</dt></dl><hr /><dl>\n", NULL);
         ap_rvputs(r, "<dt>Current Time: ",
@@ -422,10 +424,37 @@ static int status_handler(request_rec *r)
         ap_rputs("<dt>Server uptime: ", r);
         show_time(r, up_time);
         ap_rputs("</dt>\n", r);
-        ap_get_loadavg(&t);
         ap_rprintf(r, "<dt>Server load: %.2f %.2f %.2f</dt>\n",
                    t.loadavg, t.loadavg5, t.loadavg15);
     }
+    else {
+        ap_rvputs(r, ap_get_server_name(r), "\n", NULL);
+        ap_rvputs(r, "ServerVersion: ",
+                  ap_get_server_description(), "\n", NULL);
+        ap_rvputs(r, "ServerMPM: ",
+                  ap_show_mpm(), "\n", NULL);
+        ap_rvputs(r, "Server Built: ",
+                  ap_get_server_built(), "\n", NULL);
+        ap_rvputs(r, "CurrentTime: ",
+                  ap_ht_time(r->pool, nowtime, DEFAULT_TIME_FORMAT, 0),
+                             "\n", NULL);
+        ap_rvputs(r, "RestartTime: ",
+                  ap_ht_time(r->pool,
+                             ap_scoreboard_image->global->restart_time,
+                             DEFAULT_TIME_FORMAT, 0),
+                  "\n", NULL);
+        ap_rprintf(r, "ParentServerConfigGeneration: %d\n",
+                   ap_state_query(AP_SQ_CONFIG_GEN));
+        ap_rprintf(r, "ParentServerMPMGeneration: %d\n",
+                   (int)mpm_generation);
+        ap_rprintf(r, "ServerUptimeSeconds: %u\n",
+                   up_time);
+        ap_rputs("ServerUptime:", r);
+        show_time(r, up_time);
+        ap_rputs("\n", r);
+        ap_rprintf(r, "Load1: %.2f\nLoad5: %.2f\nLoad15: %.2f\n",
+                   t.loadavg, t.loadavg5, t.loadavg15);
+    }
 
     if (ap_extended_status) {
         if (short_report) {
@@ -435,6 +464,9 @@ static int status_handler(request_rec *r)
 
 #ifdef HAVE_TIMES
             /* Allow for OS/2 not having CPU stats */
+            ap_rprintf(r, "CPUUser: %g\nCPUSystem: %g\nCPUChildrenUser: %g\nCPUChildrenSystem: %g\n",
+                       tu / tick, ts / tick, tcu / tick, tcs / tick);
+
             if (ts || tu || tcu || tcs)
                 ap_rprintf(r, "CPULoad: %g\n",
                            (tu + ts + tcu + tcs) / tick / up_time * 100.);
@@ -619,7 +651,6 @@ static int status_handler(request_rec *r)
     }
 
     if (ap_extended_status && !short_report) {
-        apr_table_t *vhosts = apr_table_make(r->pool, 10);
         if (no_table_report)
             ap_rputs("<hr /><h2>Server Details</h2>\n\n", r);
         else
@@ -631,16 +662,12 @@ static int status_handler(request_rec *r)
 #endif
                      "<th>SS</th><th>Req</th>"
                      "<th>Conn</th><th>Child</th><th>Slot</th>"
-                     "<th>Client</th><th>VHost</th>"
+                     "<th>Client</th><th>Protocol</th><th>VHost</th>"
                      "<th>Request</th></tr>\n\n", r);
 
         for (i = 0; i < server_limit; ++i) {
             for (j = 0; j < thread_limit; ++j) {
-                char *escvhost;
-                long last_used;
-                const char *vlast;
-
-                ws_record = ap_get_scoreboard_worker_from_indexes(i, j);
+                ap_copy_scoreboard_worker(ws_record, i, j);
 
                 if (ws_record->access_count == 0 &&
                     (ws_record->status == SERVER_READY ||
@@ -659,22 +686,6 @@ static int status_handler(request_rec *r)
                 if (req_time < 0L)
                     req_time = 0L;
 
-                escvhost = ap_escape_html(r->pool, ws_record->vhost);
-                last_used = (long)apr_time_sec(nowtime - ws_record->last_used);
-                if (escvhost && *escvhost) {
-                    if ((vlast = apr_table_get(vhosts, escvhost)) != NULL) {
-                        long temp = atol(vlast);
-                        if (last_used < temp) {
-                            apr_table_setn(vhosts, apr_pstrdup(r->pool, escvhost),
-                                           apr_psprintf(r->pool, "%ld", last_used));
-                        }
-                    }
-                    else {
-                        apr_table_setn(vhosts, apr_pstrdup(r->pool, escvhost),
-                                           apr_psprintf(r->pool, "%ld", last_used));
-                    }
-                }
-
                 lres = ws_record->access_count;
                 my_lres = ws_record->my_access_count;
                 conn_lres = ws_record->conn_count;
@@ -754,7 +765,8 @@ static int status_handler(request_rec *r)
                                ws_record->times.tms_cutime / tick,
                                ws_record->times.tms_cstime / tick,
 #endif
-                               last_used,
+                               (long)apr_time_sec(nowtime -
+                                                  ws_record->last_used),
                                (long) req_time);
 
                     format_byte_out(r, conn_bytes);
@@ -764,13 +776,16 @@ static int status_handler(request_rec *r)
                     format_byte_out(r, bytes);
                     ap_rputs(")\n", r);
                     ap_rprintf(r,
-                               " <i>%s {%s}</i> <b>[%s]</b><br />\n\n",
+                               " <i>%s {%s}</i> <i>(%s)</i> <b>[%s]</b><br />\n\n",
                                ap_escape_html(r->pool,
                                               ws_record->client),
                                ap_escape_html(r->pool,
                                               ap_escape_logitem(r->pool,
                                                                 ws_record->request)),
-                               escvhost);
+                               ap_escape_html(r->pool,
+                                              ws_record->protocol),
+                               ap_escape_html(r->pool,
+                                              ws_record->vhost));
                 }
                 else { /* !no_table_report */
                     if (ws_record->status == SERVER_DEAD)
@@ -839,18 +854,22 @@ static int status_handler(request_rec *r)
                                 ws_record->times.tms_cutime +
                                 ws_record->times.tms_cstime) / tick,
 #endif
-                               last_used,
+                               (long)apr_time_sec(nowtime -
+                                                  ws_record->last_used),
                                (long)req_time);
 
                     ap_rprintf(r, "</td><td>%-1.1f</td><td>%-2.2f</td><td>%-2.2f\n",
                                (float)conn_bytes / KBYTE, (float) my_bytes / MBYTE,
                                (float)bytes / MBYTE);
 
-                    ap_rprintf(r, "</td><td>%s</td><td nowrap>%s</td>"
+                    ap_rprintf(r, "</td><td>%s</td><td>%s</td><td nowrap>%s</td>"
                                   "<td nowrap>%s</td></tr>\n\n",
                                ap_escape_html(r->pool,
                                               ws_record->client),
-                               escvhost,
+                               ap_escape_html(r->pool,
+                                              ws_record->protocol),
+                               ap_escape_html(r->pool,
+                                              ws_record->vhost),
                                ap_escape_html(r->pool,
                                               ap_escape_logitem(r->pool,
                                                       ws_record->request)));
@@ -859,10 +878,6 @@ static int status_handler(request_rec *r)
         } /* for (i...) */
 
         if (!no_table_report) {
-            int i;
-            const apr_array_header_t *arr = apr_table_elts(vhosts);
-            const apr_table_entry_t *elts = (const apr_table_entry_t *)arr->elts;
-
             ap_rputs("</table>\n \
 <hr /> \
 <table>\n \
@@ -881,13 +896,6 @@ static int status_handler(request_rec *r)
 <tr><th>Child</th><td>Megabytes transferred this child</td></tr>\n \
 <tr><th>Slot</th><td>Total megabytes transferred this slot</td></tr>\n \
 </table>\n", r);
-            ap_rputs("<hr />\n<table>\n\
-<tr><th>Vhost</th><th>Seconds since last used</th></tr>\n", r);
-            for (i = 0; i < arr->nelts; i++) {
-                ap_rprintf(r, "<tr><td><pre>%s</pre></td><td><pre>%27s</pre></td></tr>\n",
-                           elts[i].key, elts[i].val);
-            }
-            ap_rputs("</table>\n", r);
         }
     } /* if (ap_extended_status && !short_report) */
     else {
@@ -981,4 +989,3 @@ AP_DECLARE_MODULE(status) =
     NULL,                       /* command table */
     register_hooks              /* register_hooks */
 };
-
