@@ -82,7 +82,7 @@ argstr_to_table(apr_pool_t *p, char *str, apr_table_t *parms)
         ap_unescape_url(value);
         apr_table_set(parms, key, value);
         /*
-         ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+         ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(03230)
          "Found query arg: %s = %s", key, value);
          */
         key = apr_strtok(NULL, "&", &strtok_state);
@@ -127,7 +127,7 @@ static apr_status_t readfile_heartbeats(const char *path, apr_hash_t *servers,
             hb_server_t *server;
             char buf[4096];
             apr_size_t bsize = sizeof(buf);
-            const char *ip;
+            const char *ip, *val;
 
             apr_brigade_cleanup(tmpbb);
 
@@ -162,7 +162,7 @@ static apr_status_t readfile_heartbeats(const char *path, apr_hash_t *servers,
                 continue;
             }
 
-            ip = apr_pstrndup(pool, buf, t - buf);
+            ip = apr_pstrmemdup(pool, buf, t - buf);
             t++;
 
             server = apr_hash_get(servers, ip, APR_HASH_KEY_STRING);
@@ -180,20 +180,20 @@ static apr_status_t readfile_heartbeats(const char *path, apr_hash_t *servers,
 
             argstr_to_table(pool, apr_pstrdup(pool, t), hbt);
 
-            if (apr_table_get(hbt, "busy")) {
-                server->busy = atoi(apr_table_get(hbt, "busy"));
+            if ((val = apr_table_get(hbt, "busy"))) {
+                server->busy = atoi(val);
             }
 
-            if (apr_table_get(hbt, "ready")) {
-                server->ready = atoi(apr_table_get(hbt, "ready"));
+            if ((val = apr_table_get(hbt, "ready"))) {
+                server->ready = atoi(val);
             }
 
-            if (apr_table_get(hbt, "lastseen")) {
-                server->seen = atoi(apr_table_get(hbt, "lastseen"));
+            if ((val = apr_table_get(hbt, "lastseen"))) {
+                server->seen = atoi(val);
             }
 
-            if (apr_table_get(hbt, "port")) {
-                server->port = atoi(apr_table_get(hbt, "port"));
+            if ((val = apr_table_get(hbt, "port"))) {
+                server->port = atoi(val);
             }
 
             if (server->busy == 0 && server->ready != 0) {
@@ -342,12 +342,14 @@ static proxy_worker *find_best_hb(proxy_balancer *balancer,
     return mycandidate;
 }
 
-static apr_status_t reset(proxy_balancer *balancer, server_rec *s) {
-        return APR_SUCCESS;
+static apr_status_t reset(proxy_balancer *balancer, server_rec *s)
+{
+    return APR_SUCCESS;
 }
 
-static apr_status_t age(proxy_balancer *balancer, server_rec *s) {
-        return APR_SUCCESS;
+static apr_status_t age(proxy_balancer *balancer, server_rec *s)
+{
+    return APR_SUCCESS;
 }
 
 static const proxy_balancer_method heartbeat =
@@ -360,7 +362,7 @@ static const proxy_balancer_method heartbeat =
 };
 
 static int lb_hb_init(apr_pool_t *p, apr_pool_t *plog,
-                          apr_pool_t *ptemp, server_rec *s)
+                      apr_pool_t *ptemp, server_rec *s)
 {
     apr_size_t size;
     unsigned int num;
@@ -407,7 +409,7 @@ static void *lb_hb_create_config(apr_pool_t *p, server_rec *s)
 {
     lb_hb_ctx_t *ctx = (lb_hb_ctx_t *) apr_palloc(p, sizeof(lb_hb_ctx_t));
 
-    ctx->path = ap_server_root_relative(p, "logs/hb.dat");
+    ctx->path = ap_runtime_dir_relative(p, DEFAULT_HEARTBEAT_STORAGE);
 
     return ctx;
 }
@@ -442,7 +444,7 @@ static const char *cmd_lb_hb_storage(cmd_parms *cmd,
         return err;
     }
 
-    ctx->path = ap_server_root_relative(p, path);
+    ctx->path = ap_runtime_dir_relative(p, path);
 
     return NULL;
 }
